package com.zanxiang.game.data.serve.pojo.vo;

import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotEmpty;
import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;

/**
 * packageName com.zanxiang.game.data.serve.pojo.vo
 *
 * @author ZhangXianyu
 * @date 2024/9/2
 * @description 每日媒体留存数据VO
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class MediaActiveDataDayVO implements Serializable {

    /**
     * 游戏ID
     */
    @ApiModelProperty(value = "游戏ID")
    private Long gameId;

    /**
     * 媒体类型
     */
    @ApiModelProperty(value = "媒体类型")
    private String accountType;

    /**
     * 推广游戏名称
     */
    @ApiModelProperty(value = "游戏名称")
    private String gameName;

    /**
     * 推广游戏应用类型
     */
    @ApiModelProperty(value = "游戏类型")
    private Long gameClassify;

    /**
     * SDK来源
     */
    @ApiModelProperty(value = "SDK来源")
    private String sourceSystem;

    /**
     * 注册时间
     */
    @ApiModelProperty(value = "注册时间（消耗时间）")
    private LocalDate costDate;

    /**
     * 消耗
     */
    @ApiModelProperty(value = "消耗")
    private BigDecimal cost;

    /**
     * 注册人数
     */
    @ApiModelProperty(value = "注册人数")
    private Long regNum;

    /**
     * 首日新用户充值次数
     */
    @ApiModelProperty(value = "首日新用户充值次数")
    private Long firstNewUserAmountCount;

    /**
     * 首日新用户充值人数
     */
    @ApiModelProperty(value = "首日新用户充值人数")
    private Long firstNewUserAmountNum;

    /**
     * 首日新用户充值金额
     */
    @ApiModelProperty(value = "首日新用户充值金额")
    private BigDecimal firstNewUserAmount;

    /**
     * 老用户充值次数
     */
    @ApiModelProperty(value = "老用户充值次数")
    private Long oldUserCount;

    /**
     * 老用户充值人数
     */
    @ApiModelProperty(value = "老用户充值人数")
    private Long oldUserNum;

    @ApiModelProperty(value = "当日充值成本 = 消耗 / 新用户累计充值人数")
    private BigDecimal totalRechargeCost;

    /**
     * 老用户充值金额
     */
    @ApiModelProperty(value = "老用户充值金额")
    private BigDecimal oldUserAmount;

    /**
     * 账面充值次数
     */
    @ApiModelProperty(value = "账面充值次数")
    private Long amountCount;

    /**
     * 账面充值人数
     */
    @ApiModelProperty(value = "账面充值人数")
    private Long amountNum;

    /**
     * 账面充值金额
     */
    @ApiModelProperty(value = "账面充值金额")
    private BigDecimal amount;

    /**
     * 新用户累计充值次数
     */
    @ApiModelProperty(value = "新用户累计充值次数")
    private Long newUserTotalAmountCount;

    /**
     * 新用户累计充值人数
     */
    @ApiModelProperty(value = "新用户累计充值人数")
    private Long newUserTotalAmountNum;

    /**
     * 新用户累计充值金额
     */
    @ApiModelProperty(value = "新用户累计充值金额")
    private BigDecimal newUserTotalAmount;

    /**
     * 首日Roi
     */
    @ApiModelProperty(value = "首日Roi")
    private BigDecimal firstRoi;

    @ApiModelProperty(value = "首日付费率）")
    private BigDecimal firstAmountRate;

    @ApiModelProperty(value = "当天付费率")
    private BigDecimal todayAmountRate;

    /**
     * 当天付费ARPU
     */
    @ApiModelProperty(value = "当天付费ARPU")
    private BigDecimal todayAmountArpu;

    /**
     * 账面ARPU
     */
    @ApiModelProperty(value = "账面ARPU")
    private BigDecimal amountArpu;

    /**
     * 首日付费ARPU
     */
    @ApiModelProperty(value = "首日付费ARPU")
    private BigDecimal firstAmountArpu;

    /**
     * 账面客单价
     */
    @ApiModelProperty(value = "账面客单价")
    private BigDecimal avgAmount;

    @ApiModelProperty(value = "总回收率 = 新用户累计充值金额 / 消耗")
    private BigDecimal totalRoi;

    /**
     * 新增注册ARPU
     */
    @ApiModelProperty(value = "新增注册ARPU")
    private BigDecimal regUserArpu;

    /**
     * 新用户付费比
     */
    @ApiModelProperty(value = "新用户付费比")
    private BigDecimal newUserRate;

    /**
     * 首日客单价
     */
    @ApiModelProperty(value = "首日客单价")
    private BigDecimal firstAvgAmount;

    /**
     * 当天客单价
     */
    @ApiModelProperty(value = "当天客单价")
    private BigDecimal todayAvgAmount;

    /**
     * 账面客单价
     */
    @ApiModelProperty(value = "账面客单价")
    private BigDecimal paperAvg;

    /**
     * 复充率
     */
    @ApiModelProperty(value = "复充率")
    private BigDecimal userAgainRate;

    /**
     * 新增注册ARPU
     */
    @ApiModelProperty(value = "新增注册ARPU")
    private BigDecimal newRegArpu;

    /**
     * 首日付费ARPU
     */
    @ApiModelProperty(value = "首日付费ARPU")
    private BigDecimal firstArpu;

    /**
     * 当天付费ARPU
     */
    @ApiModelProperty(value = "当天付费ARPU")
    private BigDecimal todayArpu;

    /**
     * 账面ARPU
     */
    @ApiModelProperty(value = "账面ARPU")
    private BigDecimal paperArpu;

    @ApiModelProperty(value = "注册成本 = 消耗 / 注册人数")
    private BigDecimal regCost;

    @ApiModelProperty(value = "当日回收率")
    private BigDecimal todayRoi;

    @ApiModelProperty(value = "首日充值成本 = 消耗 / 首日新用户充值人数")
    private BigDecimal firstNewUserRechargeCost;

    @ApiModelProperty(value = "单日付费100+人数")
    private Long hundredUserNum;

    @ApiModelProperty(value = "单日付费100+成本 = 消耗 / 单日付费100+人数")
    private BigDecimal hundredUserNumCost;

    @ApiModelProperty(value = "首日创角人数")
    private Long firstRoleNum;

    @ApiModelProperty(value = "创角人数")
    private Long roleNum;

    @ApiModelProperty(value = "新用户累计创角人数")
    private Long newUserTotalRoleNum;

    @ApiModelProperty(value = "首日创角人数成本")
    private BigDecimal firstRoleNumCost;

    @ApiModelProperty(value = "创角人数成本")
    private BigDecimal roleNumCost;

    @ApiModelProperty(value = "新用户累计创角人数成本")
    private BigDecimal newUserTotalRoleNumCost;

    @ApiModelProperty(value = "首日创角率")
    private BigDecimal firstRoleNumRate;

    @ApiModelProperty(value = "创角率")
    private BigDecimal roleNumRate;

    @ApiModelProperty(value = "新用户累计创角率")
    private BigDecimal newUserTotalRoleNumRate;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da1;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da2;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da3;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da4;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da5;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da6;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da7;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da8;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da9;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da10;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da11;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da12;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da13;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da14;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da15;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da16;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da17;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da18;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da19;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da20;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da21;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da22;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da23;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da24;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da25;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da26;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da27;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da28;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da29;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da30;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da31;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da32;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da33;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da34;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da35;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da36;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da37;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da38;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da39;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da40;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da41;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da42;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da43;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da44;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da45;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da46;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da47;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da48;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da49;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da50;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da51;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da52;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da53;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da54;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da55;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da56;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da57;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da58;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da59;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da60;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da61;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da62;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da63;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da64;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da65;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da66;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da67;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da68;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da69;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da70;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da71;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da72;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da73;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da74;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da75;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da76;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da77;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da78;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da79;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da80;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da81;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da82;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da83;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da84;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da85;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da86;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da87;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da88;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da89;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String da90;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m4;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m5;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m6;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m7;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m8;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m9;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m10;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m11;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "不展示")
    private String m12;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da1Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da2Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da3Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da4Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da5Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da6Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da7Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da8Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da9Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da10Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da11Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da12Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da13Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da14Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da15Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da16Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da17Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da18Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da19Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da20Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da21Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da22Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da23Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da24Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da25Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da26Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da27Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da28Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da29Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da30Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da31Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da32Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da33Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da34Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da35Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da36Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da37Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da38Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da39Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da40Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da41Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da42Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da43Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da44Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da45Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da46Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da47Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da48Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da49Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da50Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da51Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da52Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da53Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da54Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da55Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da56Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da57Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da58Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da59Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da60Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da61Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da62Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da63Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da64Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da65Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da66Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da67Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da68Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da69Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da70Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da71Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da72Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da73Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da74Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da75Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da76Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da77Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da78Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da79Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da80Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da81Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da82Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da83Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da84Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da85Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da86Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da87Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da88Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da89Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第1-90天（展示的数据）")
    private ActiveDataTrendVO da90Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m4Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m5Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m6Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m7Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m8Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m9Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m10Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m11Trend;

    /**
     * 活跃人数/留存率
     */
    @ApiModelProperty(value = "第4-12月（展示的数据）")
    private ActiveDataTrendVO m12Trend;


}