package com.zanxiang.game.module.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.module.manage.pojo.params.GameUserChatListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameUserChatVO;
import com.zanxiang.game.module.manage.service.IGameUserChatService;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2025-03-13
 * @description :
 */
@Api(tags = {"游戏聊天记录相关接口"})
@RestController
@RequestMapping("/game/chat")
@Slf4j
public class GameUserChatController {

    @Autowired
    private IGameUserChatService gameUserChatService;

    @ApiOperation(value = "查询有聊天记录的游戏")
    @GetMapping(value = "/game/list")
    @PreAuthorize(permissionKey = "manage:gameChat:gameList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<Long, String>> listOfPage() {
        return ResultVO.ok(gameUserChatService.chatGameMap());
    }

    @ApiOperation(value = "查询游戏聊天记录")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "manage:gameChat:chatList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameUserChatVO.class)})
    public ResultVO<IPage<GameUserChatVO>> listOfPage(@Validated @RequestBody GameUserChatListParam param) {
        return ResultVO.ok(gameUserChatService.listOfPage(param));
    }
}
