package com.zanxiang.game.module.sdk.controller;

import com.zanxiang.game.module.sdk.annotation.ValidLogin;
import com.zanxiang.game.module.sdk.pojo.param.BindPhoneParam;
import com.zanxiang.game.module.sdk.pojo.param.SmsCheckParam;
import com.zanxiang.game.module.sdk.pojo.param.SmsSendParam;
import com.zanxiang.game.module.sdk.pojo.param.UserData;
import com.zanxiang.game.module.sdk.service.ISmsService;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-16
 * @description : 短信服务接口
 */
@Api(tags = "短信服务接口")
@RestController
@RequestMapping(value = "/api/sms")
public class SmsController {

    @Autowired
    private ISmsService smsService;

    @ApiOperation(value = "发送短信")
    @PostMapping("/send/msg")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> smsSend(@Validated @RequestBody SmsSendParam smsSendParam, UserData userData) {
        return smsService.smsSend(smsSendParam, userData);
    }

    @ApiOperation(value = "验证码校验")
    @PostMapping("/check/msg")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> smsCheck(@Validated @RequestBody SmsCheckParam smsCheckParam) {
        return smsService.smsCheck(smsCheckParam);
    }

    @ApiOperation(value = "获取绑定手机号")
    @GetMapping("/bind/phone")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<String, Object>> getBindPhone(@ValidLogin UserData userData) {
        return ResultVO.ok(smsService.getBindPhone(userData));
    }

    @ApiOperation(value = "绑定手机号发送验证码")
    @GetMapping("/bind/phone/send")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<String, Object>> bindPhoneSend(@RequestParam String phone, @ValidLogin UserData userData) {
        return ResultVO.ok(smsService.bindPhoneSend(phone, userData));
    }

    @ApiOperation(value = "绑定手机号验证")
    @PostMapping("/bind/phone/check")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<String, Object>> bindPhoneCheck(@Validated @RequestBody BindPhoneParam param, @ValidLogin UserData userData) {
        return ResultVO.ok(smsService.bindPhoneCheck(param, userData));
    }
}
